/************************************************************************/
/*  Routines for computing distance between 2 points on a sphere.	*/
/*  Author: Unknown							*/
/************************************************************************/

#ifndef lint
static char sccsid[] = "$Id: distance.c,v 1.2 2003/10/29 21:17:28 lombard Exp $ ";
#endif

#include <math.h>
#include "externals.h"

static	double deg_to_rad = 0.0174532;
static	double deg_to_km=111.195;

double distance 
   (VERTEX	pt1,		/* first location.			*/
    VERTEX	pt2)		/* second location.			*/
{
/* now do actual work */
    double pt1_colat,pt2_colat;
    double gc_collat(),x0,y0,z0,x1,y1,z1,x2,y2,z2;
    double del,azim;
    double del_deg, azim_deg, back_azim_deg, del_km;
    void angles();

    pt1_colat = gc_collat(pt1.lat*deg_to_rad);
    pt2_colat = gc_collat(pt2.lat*deg_to_rad);
	
    pt1.lon *= deg_to_rad;
    pt2.lon *= deg_to_rad;
    x0 = sin(pt2_colat)*cos(pt2.lon);
    y0 = sin(pt2_colat)*sin(pt2.lon);
    z0 = cos(pt2_colat);
    x1 = cos(pt1.lon)*x0+sin(pt1.lon)*y0;
    y1 = -sin(pt1.lon)*x0+cos(pt1.lon)*y0;
    z1 = z0;
    x2 = cos(pt1_colat)*x1-sin(pt1_colat)*z1;
    y2 = y1;
    z2 = cos(pt1_colat)*z1+sin(pt1_colat)*x1;

/* calculate distance between pt1 and pt2 and
    azimuth from pt1 to pt2 */

    angles(x2,y2,z2,&del_deg,&azim_deg);
    azim_deg = 180.0-azim_deg;
	
/* calculate azimuth from pt2 to pt1  */
	
    x0 = sin(pt1_colat)*cos(pt1.lon);
    y0 = sin(pt1_colat)*sin(pt1.lon);
    z0 = cos(pt1_colat);
    x1 = cos(pt2.lon)*x0+sin(pt2.lon)*y0;
    y1 = -sin(pt2.lon)*x0+cos(pt2.lon)*y0;
    z1 = z0;
    x2 = cos(pt2_colat)*x1-sin(pt2_colat)*z1;
    y2 = y1;
    z2 = cos(pt2_colat)*z1+sin(pt2_colat)*x1;
    angles(x2,y2,z2,&del_deg,&back_azim_deg);
    back_azim_deg = 180.0-back_azim_deg;

    del_km = del_deg*deg_to_km;
    pt1.lon /= deg_to_rad;
    pt2.lon /= deg_to_rad;
/*::
    printf ("dist_az -- %lf %lf %lf %lf = %lf\n", 
	    pt1.lat, pt1.lon, pt2.lat, pt2.lon, del_km);
::*/
    return (del_km);
}
	
/* function gc_collat -- given a geographic lat (in radians)
   return geocentric colat, in radians */
double gc_collat (double	lat)	/* latitude in radians.		*/
{
    double w,s,eccent=1.0/298.25;
    double pi = (((double) 4) * atan(1.0));
    w = sin(lat);
    s = ((2.0-eccent)*w + 4.0*eccent*(w*w*w))*eccent*cos(lat);
    return((pi/2.0)-lat+s);
}

/* void function angles -- find the angle theta and phi of spherical polar
   coordinate system from cartesian coords z,y,x */
void angles
   (double	x,		/* cartesian coordinate x		*/
    double	y,		/* cartesian coordinate y		*/
    double	z,		/* cartesian coordinate z		*/
    double	*theta,		/* returned spherical polar coordinate.	*/
    double	*phi)		/* returned spherical polar coordinate.	*/
{
    double pi = (((double) 4) * atan(1.0)),epsilon = 1.0e-4;
    double rad_to_deg = 180.0/pi,arg1;
    
    arg1 = sqrt(x*x+y*y);
    *theta = atan2(arg1,z);
    *phi = (fabs(x) <= epsilon && fabs(y) <= epsilon) ? 0. : atan2(y,x);
    *phi *= rad_to_deg;
    *theta *= rad_to_deg;
}
